module Gargantext.Components.GraphQL.Node where

import Data.Maybe (Maybe(..))
import Gargantext.Prelude
import Gargantext.Types (NodeType)
import Gargantext.Utils.GraphQL as GGQL
import Gargantext.Types (NodeType)
import GraphQL.Client.Alias ((:))
import GraphQL.Client.Args (Args, (=>>), OrArg(..), IgnoreArg(..))
import GraphQL.Client.Variable (Var(..))
import Type.Proxy (Proxy(..))

type Corpus =
  { id :: Int
  , name :: String
  , parent_id :: Maybe Int
  , type_id :: Int
  , node_type :: NodeType
  }

type Node =
  { id :: Int
  , name :: String
  , parent_id :: Maybe Int
  , type_id :: Int
  , node_type :: NodeType
  }

type NodeQ =
  { id :: Unit
  , name :: Unit
  , parent_id :: Unit
  , type_id :: Unit
  , node_type :: Unit
  }

nodeQ :: NodeQ
nodeQ = GGQL.getFieldsStandard (Proxy :: _ Node)

type CorpusByIdQuery =
  { corpus ::
      Args
        { id :: Var "id" Int }
        NodeQ
  }

type NodeByIdQuery =
  { node ::
      Args
        { id :: Var "id" Int }
        NodeQ
  }

getNodeByIdQuery :: NodeByIdQuery
getNodeByIdQuery =
  { node: { id: Var :: _ "id" Int } =>> nodeQ
  }

type NodesQueryArgs =
  { parentId :: OrArg IgnoreArg Int
  , containsChildId :: OrArg IgnoreArg Int
  , nodeType :: OrArg IgnoreArg NodeType
  }

type NodesQuery =
  { nodes :: Args NodesQueryArgs NodeQ }

getNodesQuery :: NodesQueryArgs -> NodesQuery
getNodesQuery args =
  { nodes: args =>> nodeQ }

-- getNodesQuery :: Maybe Int -> Maybe Int -> NodesQuery
-- getNodesQuery mParentId mChildId =
--   { nodes: { parentId: mVar mParentId
--            , containsChildId: mVar mChildId } =>>
--            -- , nodeType: Var :: _ "nodeType" (Maybe NodeType) } =>>
--       nodeQ
--   }
--   where
--     mVar Nothing = ArgL IgnoreArg
--     mVar (Just x) = ArgR x

getCorpusByIdQuery :: CorpusByIdQuery
getCorpusByIdQuery =
  { corpus: { id: Var :: _ "id" Int } =>>
      GGQL.getFieldsStandard (Proxy :: _ Corpus)
  }

getNodeParentQuery id nodeType =
  { parent:
      nodes
        :
          { containsChildId: id
          , nodeType: nodeType
          }
        =>>
          nodeQ
  }
  where
  nodes = Proxy :: Proxy "nodes"

getNodeChildrenQuery =
  { nodes:
      { parentId: Var :: _ "id" Int
      , nodeType: Var :: _ "nodeType" NodeType
      } =>>
        nodeQ
  }
